@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 # This file is part of beets.
-# Copyright 2015, Adrian Sampson.
+# Copyright 2016, Adrian Sampson.
 #
 # Permission is hereby granted, free of charge, to any person obtaining
 # a copy of this software and associated documentation files (the
@@ -13,17 +13,14 @@
 # The above copyright notice and this permission notice shall be
 # included in all copies or substantial portions of the Software.
 
-from __future__ import absolute_import, unicode_literals
+from __future__ import division, absolute_import, print_function
 
 import os
 
-import beets.library
 from beets.util import confit
 
-__version__ = '1.3.16'
-__author__ = 'Adrian Sampson <adrian@radbox.org>'
-
-Library = beets.library.Library
+__version__ = u'1.3.18'
+__author__ = u'Adrian Sampson <adrian@radbox.org>'
 
 
 class IncludeLazyConfig(confit.LazyConfig):
