@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 # This file is part of beets.
-# Copyright 2015, Adrian Sampson.
+# Copyright 2016, Adrian Sampson.
 #
 # Permission is hereby granted, free of charge, to any person obtaining
 # a copy of this software and associated documentation files (the
@@ -14,15 +14,16 @@
 # included in all copies or substantial portions of the Software.
 
 """Glue between metadata sources and the matching logic."""
-from __future__ import (division, absolute_import, print_function,
-                        unicode_literals)
+from __future__ import division, absolute_import, print_function
 
 from collections import namedtuple
+from functools import total_ordering
 import re
 
 from beets import logging
 from beets import plugins
 from beets import config
+from beets.util import as_string
 from beets.autotag import mb
 from jellyfish import levenshtein_distance
 from unidecode import unidecode
@@ -206,8 +207,8 @@ def _string_dist_basic(str1, str2):
     """
     assert isinstance(str1, unicode)
     assert isinstance(str2, unicode)
-    str1 = unidecode(str1).decode('ascii')
-    str2 = unidecode(str2).decode('ascii')
+    str1 = as_string(unidecode(str1))
+    str2 = as_string(unidecode(str2))
     str1 = re.sub(r'[^a-z0-9]', '', str1.lower())
     str2 = re.sub(r'[^a-z0-9]', '', str2.lower())
     if not str1 and not str2:
@@ -289,6 +290,7 @@ class LazyClassProperty(object):
         return self.value
 
 
+@total_ordering
 class Distance(object):
     """Keeps track of multiple distance penalties. Provides a single
     weighted distance for all penalties as well as a weighted distance
@@ -298,7 +300,7 @@ class Distance(object):
         self._penalties = {}
 
     @LazyClassProperty
-    def _weights(cls):
+    def _weights(cls):  # noqa
         """A dictionary from keys to floating-point weights.
         """
         weights_view = config['match']['distance_weights']
@@ -350,12 +352,21 @@ class Distance(object):
         # Convert distance into a negative float we can sort items in
         # ascending order (for keys, when the penalty is equal) and
         # still get the items with the biggest distance first.
-        return sorted(list_, key=lambda (key, dist): (0 - dist, key))
+        return sorted(
+            list_,
+            key=lambda key_and_dist: (-key_and_dist[1], key_and_dist[0])
+        )
+
+    def __hash__(self):
+        return id(self)
+
+    def __eq__(self, other):
+        return self.distance == other
 
     # Behave like a float.
 
-    def __cmp__(self, other):
-        return cmp(self.distance, other)
+    def __lt__(self, other):
+        return self.distance < other
 
     def __float__(self):
         return self.distance
@@ -394,7 +405,7 @@ class Distance(object):
         """
         if not isinstance(dist, Distance):
             raise ValueError(
-                '`dist` must be a Distance object, not {0}'.format(type(dist))
+                u'`dist` must be a Distance object, not {0}'.format(type(dist))
             )
         for key, penalties in dist._penalties.iteritems():
             self._penalties.setdefault(key, []).extend(penalties)
@@ -418,7 +429,7 @@ class Distance(object):
         """
         if not 0.0 <= dist <= 1.0:
             raise ValueError(
-                '`dist` must be between 0.0 and 1.0, not {0}'.format(dist)
+                u'`dist` must be between 0.0 and 1.0, not {0}'.format(dist)
             )
         self._penalties.setdefault(key, []).append(dist)
 
@@ -514,7 +525,7 @@ def album_for_mbid(release_id):
     try:
         album = mb.album_for_id(release_id)
         if album:
-            plugins.send('albuminfo_received', info=album)
+            plugins.send(u'albuminfo_received', info=album)
         return album
     except mb.MusicBrainzAPIError as exc:
         exc.log(log)
@@ -527,7 +538,7 @@ def track_for_mbid(recording_id):
     try:
         track = mb.track_for_id(recording_id)
         if track:
-            plugins.send('trackinfo_received', info=track)
+            plugins.send(u'trackinfo_received', info=track)
         return track
     except mb.MusicBrainzAPIError as exc:
         exc.log(log)
@@ -538,9 +549,9 @@ def albums_for_id(album_id):
     candidates = [album_for_mbid(album_id)]
     plugin_albums = plugins.album_for_id(album_id)
     for a in plugin_albums:
-        plugins.send('albuminfo_received', info=a)
+        plugins.send(u'albuminfo_received', info=a)
     candidates.extend(plugin_albums)
-    return filter(None, candidates)
+    return [a for a in candidates if a]
 
 
 def tracks_for_id(track_id):
@@ -548,9 +559,9 @@ def tracks_for_id(track_id):
     candidates = [track_for_mbid(track_id)]
     plugin_tracks = plugins.track_for_id(track_id)
     for t in plugin_tracks:
-        plugins.send('trackinfo_received', info=t)
+        plugins.send(u'trackinfo_received', info=t)
     candidates.extend(plugin_tracks)
-    return filter(None, candidates)
+    return [t for t in candidates if t]
 
 
 def album_candidates(items, artist, album, va_likely):
@@ -581,7 +592,7 @@ def album_candidates(items, artist, album, va_likely):
 
     # Notify subscribed plugins about fetched album info
     for a in out:
-        plugins.send('albuminfo_received', info=a)
+        plugins.send(u'albuminfo_received', info=a)
 
     return out
 
@@ -605,6 +616,6 @@ def item_candidates(item, artist, title):
 
     # Notify subscribed plugins about fetched track info
     for i in out:
-        plugins.send('trackinfo_received', info=i)
+        plugins.send(u'trackinfo_received', info=i)
 
     return out
